/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  Outdoor Thermometer Copyright (c) 2015 by Jack Botner

  led.c

  To display numbers on a 4 digit LED display. Since no segment
  encoder chip is used, we have to do the encoding of each digit
  ourself.
-----------------------------------------------------------------*/
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "led.h"

/*-----------------------------------------------------------------
  ledOutputNumber(...)

  Outputs a number to the led display. Clearing the port turns
  the corresponding segment on.

  Input: a single numeric digit, range 0..9 (ascii or binary, only
         the low-order nibble is processed.) Also the value of x'0f'
         which means blank the display.
-----------------------------------------------------------------*/

// Bit positions in the table of segments
#define SEG_A   0
#define SEG_B   1
#define SEG_C   2
#define SEG_D   3
#define SEG_E   4
#define SEG_F   5
#define SEG_G   6

void ledOutputNumber( uint8_t ucNumber, uint8_t ucDigit )
{
  // Segment A = bit 0, and so on, for each digit 0..9, blank and minus sign
  static const uint8_t ucDigTab[12] =
  {
        0b00111111, 0b00000110, 0b01011011, 0b01001111, 0b01100110,
        0b01101101, 0b01111101, 0b00000111, 0b01111111, 0b01100111,
		0b00000000, 0b01000000
  };

  uint8_t ucNibble, ucSegments;

  ucNibble = ucNumber & 0x0f;
  if ( ucNibble > 11 ) ucNibble = 10;
  ucSegments = /* ( ucNibble > 9 ) ? 0 : */ ucDigTab[ucNibble];

  if ( test_bit( ucSegments, SEG_A ) )
    clear_bit( SEG_A_PORT, SEG_A_BIT );
  else
    set_bit( SEG_A_PORT, SEG_A_BIT );

  if ( test_bit( ucSegments, SEG_B ) )
    clear_bit( SEG_B_PORT, SEG_B_BIT );
  else
    set_bit( SEG_B_PORT, SEG_B_BIT );

  if ( test_bit( ucSegments, SEG_C ) )
    clear_bit( SEG_C_PORT, SEG_C_BIT );
  else
    set_bit( SEG_C_PORT, SEG_C_BIT );

  if ( test_bit( ucSegments, SEG_D ) )
    clear_bit( SEG_D_PORT, SEG_D_BIT );
  else
    set_bit( SEG_D_PORT, SEG_D_BIT );

  if ( test_bit( ucSegments, SEG_E ) )
    clear_bit( SEG_E_PORT, SEG_E_BIT );
  else
    set_bit( SEG_E_PORT, SEG_E_BIT );

  if ( test_bit( ucSegments, SEG_F ) )
    clear_bit( SEG_F_PORT, SEG_F_BIT );
  else
    set_bit( SEG_F_PORT, SEG_F_BIT );

  if ( test_bit( ucSegments, SEG_G ) )
    clear_bit( SEG_G_PORT, SEG_G_BIT );
  else
    set_bit( SEG_G_PORT, SEG_G_BIT );

  if ( ucDigit == 2 )
    clear_bit( SEG_DP_PORT, SEG_DP_BIT );
  else
    set_bit( SEG_DP_PORT, SEG_DP_BIT );
}

/*---------------------------------------------------------------------
  ledSelectDigit(...)
  
  Selects a digit of the led display, by turning on the requested
  driver transistor.
  
  Input: a single numeric digit, range 0..3. 0 drives the highest 
         order digit, 3 drives the lowest order digit.
---------------------------------------------------------------------*/
void ledSelectDigit( uint8_t cDigit )
{
  if ( !cDigit )
    clear_bit( PORTC, PC5 );
  else
    set_bit( PORTC, PC5 );

  if ( cDigit == 1 )
    clear_bit( PORTC, PC4 );
  else
    set_bit( PORTC, PC4 );

  if ( cDigit == 2 )
    clear_bit( PORTC, PC3 );
  else
    set_bit( PORTC, PC3 );

  if ( cDigit == 3 )
    clear_bit( PORTC, PC2 );
  else
    set_bit( PORTC, PC2 );
}

